@@ -14,11 +14,14 @@
 # limitations under the License.
 
 import asyncio
+import inspect
 import unittest
 
+import pytest
+
 import six
 
-from tenacity import RetryError
+from tenacity import AsyncRetrying, RetryError
 from tenacity import _asyncio as tasyncio
 from tenacity import retry, stop_after_attempt
 from tenacity.tests.test_tenacity import NoIOErrorAfterCount, current_time_ms
@@ -34,6 +37,11 @@ def asynctest(callable_):
     return wrapper
 
 
+async def _async_function(thing):
+    await asyncio.sleep(0.00001)
+    return thing.go()
+
+
 @retry
 async def _retryable_coroutine(thing):
     await asyncio.sleep(0.00001)
@@ -53,6 +61,26 @@ class TestAsync(unittest.TestCase):
         await _retryable_coroutine(thing)
         assert thing.counter == thing.count
 
+    @asynctest
+    async def test_iscoroutinefunction(self):
+        assert asyncio.iscoroutinefunction(_retryable_coroutine)
+        assert inspect.iscoroutinefunction(_retryable_coroutine)
+
+    @asynctest
+    async def test_retry_using_async_retying(self):
+        thing = NoIOErrorAfterCount(5)
+        retrying = AsyncRetrying()
+        await retrying(_async_function, thing)
+        assert thing.counter == thing.count
+
+    @asynctest
+    async def test_retry_using_async_retying_legacy_method(self):
+        thing = NoIOErrorAfterCount(5)
+        retrying = AsyncRetrying()
+        with pytest.warns(DeprecationWarning):
+            await retrying.call(_async_function, thing)
+        assert thing.counter == thing.count
+
     @asynctest
     async def test_stop_after_attempt(self):
         thing = NoIOErrorAfterCount(2)
@@ -64,6 +92,10 @@ class TestAsync(unittest.TestCase):
     def test_repr(self):
         repr(tasyncio.AsyncRetrying())
 
+    def test_retry_attributes(self):
+        assert hasattr(_retryable_coroutine, 'retry')
+        assert hasattr(_retryable_coroutine, 'retry_with')
+
     @asynctest
     async def test_attempt_number_is_correct_for_interleaved_coroutines(self):
 
