@@ -14,7 +14,6 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 import logging
-import os
 import re
 import sys
 import time
@@ -34,8 +33,13 @@ from tenacity.compat import make_retry_state
 
 
 class TestBase(unittest.TestCase):
+
     def test_repr(self):
-        repr(tenacity.BaseRetrying())
+        class ConcreteRetrying(tenacity.BaseRetrying):
+            def __call__(self):
+                pass
+
+        repr(ConcreteRetrying())
 
 
 class TestStopConditions(unittest.TestCase):
@@ -135,7 +139,7 @@ class TestStopConditions(unittest.TestCase):
         def failing():
             raise NotImplementedError()
         with pytest.raises(RetryError):
-            retrying.call(failing)
+            retrying(failing)
 
     def test_stop_func_with_retry_state(self):
         def stop_func(retry_state):
@@ -283,7 +287,7 @@ class TestWaitConditions(unittest.TestCase):
                      stop=tenacity.stop_after_attempt(r_attempts),
                      reraise=True)
         with reports_deprecation_warning():
-            self.assertRaises(Exception, r.call, dying)
+            self.assertRaises(Exception, r, dying)
         self.assertEqual(r_attempts - 1, len(captures))
         self.assertTrue(all([r.failed for r in captures]))
 
@@ -451,14 +455,14 @@ class TestWaitConditions(unittest.TestCase):
 
         retrying1 = Retrying(wait=wait1, stop=tenacity.stop_after_attempt(4))
         with reports_deprecation_warning():
-            self.assertRaises(Exception, lambda: retrying1.call(dying))
+            self.assertRaises(Exception, lambda: retrying1(dying))
         self.assertEqual([t[0] for t in wait1.calls], [1, 2, 3])
         # This assumes that 3 iterations complete within 1 second.
         self.assertTrue(all(t[1] < 1 for t in wait1.calls))
 
         retrying2 = Retrying(wait=wait2, stop=tenacity.stop_after_attempt(4))
         with reports_deprecation_warning():
-            self.assertRaises(Exception, lambda: retrying2.call(dying))
+            self.assertRaises(Exception, lambda: retrying2(dying))
         self.assertEqual([t[0] for t in wait2.calls], [1, 2, 3])
         # This assumes that 3 iterations complete within 1 second.
         self.assertTrue(all(t[1] < 1 for t in wait2.calls))
@@ -494,7 +498,7 @@ class TestWaitConditions(unittest.TestCase):
         def returnval():
             return 123
         try:
-            retrying.call(returnval)
+            retrying(returnval)
         except ExtractCallState as err:
             retry_state = err.args[0]
         self.assertIs(retry_state.fn, returnval)
@@ -502,13 +506,13 @@ class TestWaitConditions(unittest.TestCase):
         self.assertEqual(retry_state.kwargs, {})
         self.assertEqual(retry_state.outcome.result(), 123)
         self.assertEqual(retry_state.attempt_number, 1)
-        self.assertGreater(retry_state.outcome_timestamp,
-                           retry_state.start_time)
+        self.assertGreaterEqual(retry_state.outcome_timestamp,
+                                retry_state.start_time)
 
         def dying():
             raise Exception("Broken")
         try:
-            retrying.call(dying)
+            retrying(dying)
         except ExtractCallState as err:
             retry_state = err.args[0]
         self.assertIs(retry_state.fn, dying)
@@ -516,8 +520,8 @@ class TestWaitConditions(unittest.TestCase):
         self.assertEqual(retry_state.kwargs, {})
         self.assertEqual(str(retry_state.outcome.exception()), 'Broken')
         self.assertEqual(retry_state.attempt_number, 1)
-        self.assertGreater(retry_state.outcome_timestamp,
-                           retry_state.start_time)
+        self.assertGreaterEqual(retry_state.outcome_timestamp,
+                                retry_state.start_time)
 
 
 class TestRetryConditions(unittest.TestCase):
@@ -598,7 +602,7 @@ class TestRetryConditions(unittest.TestCase):
     def test_retry_try_again(self):
         self._attempts = 0
         Retrying(stop=tenacity.stop_after_attempt(5),
-                 retry=tenacity.retry_never).call(self._raise_try_again)
+                 retry=tenacity.retry_never)(self._raise_try_again)
         self.assertEqual(3, self._attempts)
 
     def test_retry_try_again_forever(self):
@@ -608,7 +612,7 @@ class TestRetryConditions(unittest.TestCase):
         r = Retrying(stop=tenacity.stop_after_attempt(5),
                      retry=tenacity.retry_never)
         self.assertRaises(tenacity.RetryError,
-                          r.call,
+                          r,
                           _r)
         self.assertEqual(5, r.statistics['attempt_number'])
 
@@ -1048,7 +1052,7 @@ class TestDecoratorWrapper(unittest.TestCase):
         def failing():
             raise NotImplementedError()
         with pytest.raises(RetryError):
-            retrying.call(failing)
+            retrying(failing)
 
 
 class TestBeforeAfterAttempts(unittest.TestCase):
@@ -1143,7 +1147,7 @@ class TestBeforeAfterAttempts(unittest.TestCase):
 
         self.assertEqual(self.slept, 2)
 
-    def test_before_sleep_log_raises(self):
+    def _before_sleep_log_raises(self, get_call_fn):
         thing = NoIOErrorAfterCount(2)
         logger = logging.getLogger(self.id())
         logger.propagate = False
@@ -1155,7 +1159,7 @@ class TestBeforeAfterAttempts(unittest.TestCase):
             retrying = Retrying(wait=tenacity.wait_fixed(0.01),
                                 stop=tenacity.stop_after_attempt(3),
                                 before_sleep=_before_sleep)
-            retrying.call(thing.go)
+            get_call_fn(retrying)(thing.go)
         finally:
             logger.removeHandler(handler)
 
@@ -1166,6 +1170,12 @@ class TestBeforeAfterAttempts(unittest.TestCase):
         self.assertRegexpMatches(fmt(handler.records[0]), etalon_re)
         self.assertRegexpMatches(fmt(handler.records[1]), etalon_re)
 
+    def test_before_sleep_log_raises(self):
+        self._before_sleep_log_raises(lambda x: x)
+
+    def test_before_sleep_log_raises_deprecated_call(self):
+        self._before_sleep_log_raises(lambda x: x.call)
+
     def test_before_sleep_log_raises_with_exc_info(self):
         thing = NoIOErrorAfterCount(2)
         logger = logging.getLogger(self.id())
@@ -1180,14 +1190,14 @@ class TestBeforeAfterAttempts(unittest.TestCase):
             retrying = Retrying(wait=tenacity.wait_fixed(0.01),
                                 stop=tenacity.stop_after_attempt(3),
                                 before_sleep=_before_sleep)
-            retrying.call(thing.go)
+            retrying(thing.go)
         finally:
             logger.removeHandler(handler)
 
         etalon_re = re.compile(r"^Retrying .* in 0\.01 seconds as it raised "
                                r"(IO|OS)Error: Hi there, I'm an IOError\.{0}"
                                r"Traceback \(most recent call last\):{0}"
-                               r".*$".format(os.linesep),
+                               r".*$".format('\n'),
                                flags=re.MULTILINE)
         self.assertEqual(len(handler.records), 2)
         fmt = logging.Formatter().format
@@ -1209,7 +1219,7 @@ class TestBeforeAfterAttempts(unittest.TestCase):
             retrying = Retrying(wait=tenacity.wait_fixed(0.01),
                                 stop=tenacity.stop_after_attempt(3),
                                 retry=_retry, before_sleep=_before_sleep)
-            retrying.call(thing.go)
+            retrying(thing.go)
         finally:
             logger.removeHandler(handler)
 
@@ -1421,6 +1431,81 @@ class TestContextManager(unittest.TestCase):
         self.assertRaises(CustomError, test)
 
 
+class TestInvokeAsCallable:
+    """Test direct invocation of Retrying as a callable."""
+
+    @staticmethod
+    def invoke(retry, f):
+        """
+        Invoke Retrying logic.
+
+        Wrapper allows testing different call mechanisms in test sub-classes.
+        """
+        return retry(f)
+
+    def test_retry_one(self):
+        def f():
+            f.calls.append(len(f.calls) + 1)
+            if len(f.calls) <= 1:
+                raise Exception("Retry it!")
+            return 42
+        f.calls = []
+
+        retry = Retrying()
+        assert self.invoke(retry, f) == 42
+        assert f.calls == [1, 2]
+
+    def test_on_error(self):
+        class CustomError(Exception):
+            pass
+
+        def f():
+            f.calls.append(len(f.calls) + 1)
+            if len(f.calls) <= 1:
+                raise CustomError("Don't retry!")
+            return 42
+        f.calls = []
+
+        retry = Retrying(retry=tenacity.retry_if_exception_type(IOError))
+        with pytest.raises(CustomError):
+            self.invoke(retry, f)
+        assert f.calls == [1]
+
+    def test_retry_error(self):
+        def f():
+            f.calls.append(len(f.calls) + 1)
+            raise Exception("Retry it!")
+        f.calls = []
+
+        retry = Retrying(stop=tenacity.stop_after_attempt(2))
+        with pytest.raises(RetryError):
+            self.invoke(retry, f)
+        assert f.calls == [1, 2]
+
+    def test_reraise(self):
+        class CustomError(Exception):
+            pass
+
+        def f():
+            f.calls.append(len(f.calls) + 1)
+            raise CustomError("Retry it!")
+        f.calls = []
+
+        retry = Retrying(reraise=True, stop=tenacity.stop_after_attempt(2))
+        with pytest.raises(CustomError):
+            self.invoke(retry, f)
+        assert f.calls == [1, 2]
+
+
+class TestInvokeViaLegacyCallMethod(TestInvokeAsCallable):
+    """Retrying.call() method should work the same as Retrying.__call__()."""
+
+    @staticmethod
+    def invoke(retry, f):
+        with reports_deprecation_warning():
+            return retry.call(f)
+
+
 class TestRetryException(unittest.TestCase):
 
     def test_retry_error_is_pickleable(self):
@@ -1479,5 +1564,50 @@ def reports_deprecation_warning():
         warnings.filters = oldfilters
 
 
+class TestMockingSleep():
+    RETRY_ARGS = dict(
+        wait=tenacity.wait_fixed(0.1),
+        stop=tenacity.stop_after_attempt(5),
+    )
+
+    def _fail(self):
+        raise NotImplementedError()
+
+    @retry(**RETRY_ARGS)
+    def _decorated_fail(self):
+        self._fail()
+
+    @pytest.fixture()
+    def mock_sleep(self, monkeypatch):
+        class MockSleep(object):
+            call_count = 0
+
+            def __call__(self, seconds):
+                self.call_count += 1
+
+        sleep = MockSleep()
+        monkeypatch.setattr(tenacity.nap.time, "sleep", sleep)
+        yield sleep
+
+    def test_call(self, mock_sleep):
+        retrying = Retrying(**self.RETRY_ARGS)
+        with pytest.raises(RetryError):
+            retrying.call(self._fail)
+        assert mock_sleep.call_count == 4
+
+    def test_decorated(self, mock_sleep):
+        with pytest.raises(RetryError):
+            self._decorated_fail()
+        assert mock_sleep.call_count == 4
+
+    def test_decorated_retry_with(self, mock_sleep):
+        fail_faster = self._decorated_fail.retry_with(
+            stop=tenacity.stop_after_attempt(2),
+        )
+        with pytest.raises(RetryError):
+            fail_faster()
+        assert mock_sleep.call_count == 1
+
+
 if __name__ == '__main__':
     unittest.main()
