@@ -18,8 +18,7 @@ import sys
 from tenacity import BaseRetrying
 from tenacity import DoAttempt
 from tenacity import DoSleep
-from tenacity import NO_RESULT
-from tenacity import _utils
+from tenacity import RetryCallState
 
 from tornado import gen
 
@@ -33,28 +32,22 @@ class TornadoRetrying(BaseRetrying):
         self.sleep = sleep
 
     @gen.coroutine
-    def call(self, fn, *args, **kwargs):
+    def __call__(self, fn, *args, **kwargs):
         self.begin(fn)
 
-        result = NO_RESULT
-        exc_info = None
-        start_time = _utils.now()
-
+        retry_state = RetryCallState(
+            retry_object=self, fn=fn, args=args, kwargs=kwargs)
         while True:
-            do = self.iter(result=result, exc_info=exc_info,
-                           start_time=start_time)
+            do = self.iter(retry_state=retry_state)
             if isinstance(do, DoAttempt):
                 try:
                     result = yield fn(*args, **kwargs)
-                    exc_info = None
-                    continue
                 except BaseException:
-                    result = NO_RESULT
-                    exc_info = sys.exc_info()
-                    continue
+                    retry_state.set_exception(sys.exc_info())
+                else:
+                    retry_state.set_result(result)
             elif isinstance(do, DoSleep):
-                result = NO_RESULT
-                exc_info = None
+                retry_state.prepare_for_next_attempt()
                 yield self.sleep(do)
             else:
                 raise gen.Return(do)
